﻿using System.Collections.Generic;
using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우 뷰 모델
    /// </summary>
    public class  MainWindowViewModel : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 직원 리스트
        /// </summary>
        private List<Employee> employeeList = null;

        /// <summary>
        /// 선택 직원
        /// </summary>
        private Employee selectedEmployee = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 직원 리스트 - EmployeeList

        /// <summary>
        /// 직원 리스트
        /// </summary>
        public List<Employee> EmployeeList
        {
            get
            {
                return this.employeeList;
            }
            set
            {
                if(this.employeeList == value)
                {
                    return;
                }

                this.employeeList = value;

                FirePropertyChangedEvent("EmployeeList");
            }
        }

        #endregion
        #region 선택 직원 - SelectedEmployee

        /// <summary>
        /// 선택 직원
        /// </summary>
        public Employee SelectedEmployee
        {
            get
            {
                return this.selectedEmployee;
            }
            set
            {
                if(this.selectedEmployee == value)
                {
                    return;
                }

                this.selectedEmployee = value;

                FirePropertyChangedEvent("SelectedEmployee");
            }
        }

        #endregion
        #region 선택 변경시 명령 - SelectionChangedCommand
 
        /// <summary>
        /// 선택 변경시 명령
        /// </summary>
        public RelayCommand SelectionChangedCommand { get; set; }

        #endregion
 
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindowViewModel()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindowViewModel()
        {
            List<Employee> employeeList = new List<Employee>();

            employeeList.Add
            (
                new Employee()
                {
                    EmployeeID = 1,
                    FirstName  = "John",
                    LastName   = "Dow",
                    Title      = "Accountant",
                    Department = "Payroll"
                }
            );

            employeeList.Add
            (
                new Employee()
                {
                    EmployeeID = 2,
                    FirstName  = "Jane",
                    LastName   = "Austin",
                    Title      = "Accountant Executive",
                    Department = "Employee Management"
                }
            );

            employeeList.Add
            (
                new Employee()
                {
                    EmployeeID = 3,
                    FirstName  = "Ralph",
                    LastName   = "Emmerson",
                    Title      = "QA Manager",
                    Department = "Product Development"
                }
            );

            employeeList.Add
            (
                new Employee()
                {
                    EmployeeID = 4,
                    FirstName  = "Patrick",
                    LastName   = "Fitzgerald",
                    Title      = "QA Manager",
                    Department = "Product Development"
                }
            );

            employeeList.Add
            (
                new Employee()
                {
                    EmployeeID = 5,
                    FirstName  = "Charles",
                    LastName   = "Dickens",
                    Title      = "QA Manager",
                    Department = "Product Development"
                }
            );

            EmployeeList = employeeList;

            SelectionChangedCommand = new RelayCommand(SelectionChangedCommand_Executed);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 변경시 명령 실행시 처리하기 - SelectionChangedCommand_Executed(parameter)

        /// <summary>
        /// 선택 변경시 명령 실행시 처리하기
        /// </summary>
        /// <param name="parameter">매개 변수</param>
        public void SelectionChangedCommand_Executed(object parameter)
        {
            SelectedEmployee = parameter as Employee;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected void FirePropertyChangedEvent(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}